"""
:Project: Cruncher
:Contents: __init__.py
:copyright: © 2019-2022 Daniel Morell
:license: MIT, see LICENSE for more details.
:Author: Daniel Morell
"""
# Standard Library Imports
import os

# Package Imports
import click

# Local Imports
from .core import CrunchHandler, OUTPUT_FILE_FORMATS, ERROR_HANDLING

__version__ = '0.5.0'


@click.command()
@click.option(
    '-i', '--image', 'image',
    type=click.Path(exists=True, readable=True, file_okay=True),
    help='The absolute path to the image to be crunched.'
)
@click.option(
    '-d', '--directory', 'directory',
    type=click.Path(exists=True, readable=True, dir_okay=True),
    default=os.getcwd(),
    show_default='current directory',
    help='The absolute path to the directory of images to be crunched.'
)
@click.option(
    '-o', '--output', 'output',
    type=click.Path(),
    default=None,
    show_default='current directory /crunched',
    help='The the directory to place the crunched images in.'
)
@click.option(
    '-f', '--format', 'file_format',
    type=click.Choice(OUTPUT_FILE_FORMATS, case_sensitive=False),
    default='JPEG',
    show_default=True,
    help='The output image format of the final crunched image.'
)
@click.option(
    '-q', '--quality', 'quality',
    type=click.IntRange(1, 100, clamp=True),
    default=80,
    show_default=True,
    help='The quality of the image after it is crunched. [range: 1 - 100]'
)
@click.option(
    '-s', '--size', 'size',
    type=(int, int),
    default=(None, None),
    help='The pixel width / height of the image after it is crunched. [Format: -S <WIDTH HEIGHT>]'
)
@click.option(
    '-a', '--append', 'append',
    type=str,
    default=None,
    help='Append a string to the filename.'
)
@click.option(
    '--keep-orientation', 'orientation',
    is_flag=True,
    help='Include this flag to keep the original image orientation. I.e. if '
         '--size is landscape all portrait images will be resized as portrait, '
         'and all landscape will be resized as landscape. If this parameter is '
         'not included portrait images will become landscape, and vice versa.'
)
@click.option(
    '--keep-aspect', 'aspect',
    is_flag=True,
    help='Include this flag to keep the original image aspect ratio. If this '
         'parameter is included the image will be scaled but the aspect '
         'ratio will be retained. This should be used with --keep-orientation '
         'otherwise the aspect ratio may be scaled as landscape when the '
         'original is portrait.'
)
@click.option(
    '-m', '--keep-metadata', 'metadata',
    is_flag=True,
    help='Include this flag to keep the image meta/exif. It is removed by default.'
)
@click.option(
    '-v', '--versions', 'nversions',
    default=1,
    show_default=True,
    help='The number of versions to create for each image. If this is set to more '
         'than one you will be prompted to enter --format, --quality, --size, '
         '--append, --ignore-orientation, and --keep-metadata.'
)
@click.option(
    '-r', '--recursive', 'recursive',
    is_flag=True,
    help='Get images from sub directories also. Only applicable if --directory '
         'is used.'
)
@click.option(
    '-c', '--config', 'config',
    type=click.Path(),
    default=None,
    help='Specify a JSON file with your settings, this will override all other settings.'
)
@click.option(
    '-V', '--version', 'version',
    is_flag=True,
    default=None,
    help='Returns the version of Cruncher.'
)
# @click.option(
#     '-e', '--errors', 'errors',
#     type=click.Choice(ERROR_HANDLING, case_sensitive=False),
#     default='errors',
#     show_default=True,
#     help='Specify how Cruncher should handle errors.'
# )
def cli(image, directory, output, file_format, quality, size, append,
        orientation, aspect, metadata, nversions, recursive, config, version):
    """
    Cruncher 0.3.0

    This is a simple yet powerful CLI image optimization wrapper for the
    Python Image Library fork Pillow. Cruncher takes images and scales them to
    the specified size and quality.
    """
    if version is not None:
        click.echo('Cruncher ' + __version__)
        return

    cruncher = CrunchHandler(image=image, directory=directory, output=output, file_format=file_format,
                             quality=quality, size=size, append=append,  orientation=orientation,
                             metadata=metadata, aspect=aspect, nversions=nversions, recursive=recursive,
                             config=config)
    cruncher.run_cruncher()

    # Calculate Stats
    stats = cruncher.get_stats()
    click.echo(
        f"\nDone :: All images have been crunched."
        f"\n\nStats ::"
        f"\nImages crunched: {stats['images']}"
        f"\nVersions:        {stats['versions']}"
        f"\nImages created:  {stats['new_images']}"
        f"\nInput Size:      {stats['input_bytes'][0]} {stats['input_bytes'][1]}"
        f"\nOutput Size:     {stats['output_bytes'][0]} {stats['output_bytes'][1]}"
        f"\nPercent Reduced: {stats['percent_gain']}%"
        f"\nAverage Reduce:  {stats['average_gain']}%"
    )

    cruncher.print_messages()


if __name__ == '__main__':
    cli()



"""
:Project: Cruncher
:Contents: cruncher.py
:copyright: © 2019-2022 Daniel Morell
:license: MIT, see LICENSE for more details.
:Author: Daniel Morell
"""
# Standard Library Imports
import os

# Package Imports
from PIL import Image, ImageCms, ImageEnhance
import click

# Local Imports
from .utils import calculate_temperature_change


class CruncherBase:

    def __init__(self, mode, path, output, image_path, version, file_format, settings):
        self.skip_image = False
        self.image = None
        self.mode = mode
        self.path = path
        self.output = output
        self.image_path = image_path
        self.version = version
        self.format = file_format
        self.settings = settings
        self.filename = self.generate_filename(os.path.split(image_path)[1], version)
        self.new_path = self.image_output_path(image_path)
        self.exif = b''
        self.output_bytes = 0
        self.messages = []

        # Open Image
        self.open_image()  # Step 1

        if self.skip_image:
            return

        # Run conversions, corrections and resizing
        self.resize()
        self.convert_rgb()
        self.convert_icc_profile()
        self.enhance()

        if self.skip_image:
            return

        # Crunch the image
        self.crunch_image()  # needs o call self._save()

        self.get_output_kb()

    def image_output_path(self, image_path):
        """
        Calculate the output path for the image.

        :param image_path: Image input path.
        :return: Image output path.
        """
        if self.mode == 'img':
            return os.path.join(self.output, self.filename)
        relative_path = os.path.relpath(image_path, self.path)
        relative_path = os.path.join(os.path.split(relative_path)[0], self.filename)
        return os.path.join(self.output, relative_path)

    def crunch_image(self):
        pass

    def generate_filename(self, filename, version):
        if not version['append']:
            version['append'] = ''
        filename = filename.split('.')
        del (filename[-1])
        filename = '.'.join(filename)
        return f"{filename}{version['append']}.{self.format}"

    def resize(self):
        """
        The `resize()` function changes the size of an image without squashing or stretching
        as the PIL.Image.resize() function does.
        """

        image = self.image
        size = (self.version['width'], self.version['height'])
        version = self.version

        if size == (None, None):
            return image

        old_orientation = image.width / image.height
        new_orientation = size[0] / size[1]

        if (version['orientation']
                and (old_orientation <= 1 < new_orientation or old_orientation >= 1 > new_orientation)):
            # if one is horizontal and the other vertical flip the orientation
            size = (size[1], size[0])
            new_orientation = size[0] / size[1]

        if size == (image.width, image.height):
            self.image = image
            return

        if version['aspect']:
            image.thumbnail(size, Image.LANCZOS)
            self.image = image
            return

        if new_orientation < old_orientation:
            v_width = image.width - (image.height / size[1] * size[0])
            v_height = image.height
            left = v_width / 2
            top = 0
            right = image.width - left
            bottom = v_height

        else:
            v_width = image.width
            v_height = image.height - (image.width / size[0] * size[1])
            left = 0
            top = v_height / 2
            right = v_width
            bottom = image.height - top

        box = (left, top, right, bottom)
        try:
            resized = image.resize(size, Image.LANCZOS, box)
        except ValueError:
            self.messages.append(f'\n Image: {self.image_path} skipped because of resize error')
            self.skip_image = True
            return

        self.image = resized

    def calculate_sampling(self, options, base):
        if self.version['subsampling']:
            return self.version['subsampling']
        divisor = (100 - base) / len(options)
        raw_sampling = ((self.version['quality'] - base) / divisor) - 1
        if raw_sampling < 0:
            return 0
        return round(raw_sampling)

    def convert_rgb(self):
        self.image = self.image.convert('RGB')

    def convert_icc_profile(self):
        """
        Converts image colors based on the input and output ICC profiles.

        :since: v0.2
        :return:
        """
        try:
            if (self.version.get('icc_conversion')
                    and self.settings.get('icc_conversions').get(self.version['icc_conversion'])):
                icc = self.settings['icc_conversions'][self.version['icc_conversion']]
                self.image = ImageCms.profileToProfile(
                    self.image,
                    inputProfile=icc.get('input_icc'),
                    outputProfile=icc.get('output_icc'),
                    renderingIntent=0,
                    outputMode=icc.get('mode')
                )
        except ImageCms.PyCMSError as e:
            self.messages.append(f'Image ICC profile conversion failed: {self.image_path}')
            self.skip_image = True

    def enhance(self):
        """
        This method enhances the image based on the settings in the JSON `enhance`
        object. All enhance properties can be values between -100 and 100.
        """
        if not self.version.get('enhance'):
            return

        enhancements = self.version.get('enhance')
        for enhancement, value in enhancements.items():
            if enhancement == 'brightness' and value != 1:
                self.image = ImageEnhance.Brightness(self.image).enhance(value / 100 + 1)
            if enhancement == 'color' and value != 1:
                self.image = ImageEnhance.Color(self.image).enhance(value / 100 + 1)
            if enhancement == 'contrast' and value != 1:
                self.image = ImageEnhance.Contrast(self.image).enhance(value / 100 + 1)
            if enhancement == 'sharpness' and value != 1:
                self.image = ImageEnhance.Sharpness(self.image).enhance(value / 100 + 1)
            if enhancement == 'temperature':
                self.image = self.temperature(value)

    def temperature(self, temperature):
        """
        Adjust the image temperature on a scale from -100 to 100.
        :param temperature: int
        :return: Image
        """
        r, g, b = calculate_temperature_change(temperature)
        matrix = (r / 255.0, 0.0, 0.0, 0.0,
                  0.0, g / 255.0, 0.0, 0.0,
                  0.0, 0.0, b / 255.0, 0.0)
        return self.image.convert('RGB', matrix)

    def get_transparency(self, default=None):
        transparency = default
        if self.image.info.get('transparency'):
            transparency = self.image.info.get('transparency')
        return transparency

    def get_output_kb(self):
        """
        Get the output file size, and save to `self.output_bytes`.
        """
        self.output_bytes = os.stat(self.new_path).st_size

    def open_image(self):
        try:
            self.image = Image.open(self.image_path)
        except Image.DecompressionBombError as e:
            if click.confirm(f'\n{e}\nPath: {self.image_path}\nDo you want to try to crunch this image? '):
                Image.MAX_IMAGE_PIXELS = None
            else:
                self.skip_image = True
                self.messages.append(f'Image: {self.image_path} skipped because of decompression bomb error')
                return
        self.image = Image.open(self.image_path)
        if self.version['metadata'] and self.image.info.get('exif'):
            self.exif = self.image.info.get('exif')

    def _save(self, *args, **kwargs):
        self.image.save(*args, **kwargs)


class GIFCruncher(CruncherBase):

    def __init__(self, mode, path, output, image_path, version, settings):
        super().__init__(mode, path, output, image_path, version, "gif", settings)

    def crunch_image(self):
        transparency = self.get_transparency()
        args = [self.new_path, "GIF"]
        kwargs = {'optimize': True, 'transparency': transparency}
        self._save(self.image, *args, **kwargs)


class JPEGCruncher(CruncherBase):
    """
    Quality Based Sampling
        0 - 70: 4:2:0
        71 - 89: 4:2:2
        90 - 100: 4:4:4
    """

    def __init__(self, mode, path, output, image_path, version, settings):
        super().__init__(mode, path, output, image_path, version, "jpg", settings)

    def crunch_image(self):
        sampling = self.calculate_sampling([0, 1, 2], 40)
        args = [self.new_path, "JPEG"]
        kwargs = {
            'quality': self.version['quality'],
            'sampling': sampling,
            'optimize': True,
            'progressive': True,
            'exif': self.exif,
            'icc_profile': self.image.info.get('icc_profile')
        }
        try:
            self._save(*args, **kwargs)
        except IOError:
            self.image = self.image.convert('RGB')
            self._save(*args, **kwargs)


class JPEG2000Cruncher(CruncherBase):

    def __init__(self, mode, path, output, image_path, version, settings):
        super().__init__(mode, path, output, image_path, version, "jp2", settings)

    def crunch_image(self):
        image = Image.open(self.image_path)
        args = [self.new_path, "JPEG2000"]
        kwargs = {
            'quality_mode': 'dB',
            'quality_layers': self.version['quality'],
            'progressive': True
        }
        self._save(image, *args, **kwargs)


class PNGCruncher(CruncherBase):

    def __init__(self, mode, path, output, image_path, version, settings):
        super().__init__(mode, path, output, image_path, version, "png", settings)

    def crunch_image(self):
        image = Image.open(self.image_path)
        transparency = self.get_transparency()
        args = [self.new_path, "PNG"]
        kwargs = {
            'optimize': True,
            'exif': self.exif,
            'transparency': transparency
        }
        try:
            self._save(image, *args, **kwargs)
        except IOError:
            image = image.convert('RGB')
            self._save(image, *args, **kwargs)


class WebPCruncher(CruncherBase):

    def __init__(self, mode, path, output, image_path, version, settings):
        super().__init__(mode, path, output, image_path, version, "webp", settings)

    def crunch_image(self):
        image = Image.open(self.image_path)
        args = [self.new_path, "WEBP"]
        kwargs = {
            'quality': (100 - self.version['quality']),
            'method': 0,
            'exif': self.exif
        }
        self._save(image, *args, **kwargs)



"""
:Project: Cruncher
:Contents: setup.py
:copyright: © 2019-2022 Daniel Morell
:license: MIT, see LICENSE for more details.
:Author: Daniel Morell
"""

import io
import re
from setuptools import setup

with io.open("cruncher/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r"__version__ = '(.*?)'", f.read()).group(1)

with io.open("README.md", "rt", encoding='utf8') as fh:
    readme = fh.read()

setup(
    name='cruncher',
    version=version,
    license='GPLv3+',
    url='https://github.com/danielmorell/cruncher',
    project_urls={
        "Source Code": "https://github.com/danielmorell/cruncher",
        "Issue tracker": "https://github.com/danielmorell/cruncher/issues",
    },
    author='Daniel Morell',
    author_email='office@carintek.com',
    description="Simple CLI to optimize images for the web.",
    long_description=readme,
    long_description_content_type="text/markdown",
    py_modules=['cruncher'],
    packages=['cruncher'],
    install_requires=[
        'click==8.1.*',
        'pillow==9.2.*',
    ],
    include_package_data=True,
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    entry_points='''
        [console_scripts]
        cruncher=cruncher:cli
    ''',
)



"""
:Project: Cruncher
:Contents: core.py
:copyright: © 2019-2022 Daniel Morell
:license: MIT, see LICENSE for more details.
:Author: Daniel Morell
"""
# Standard Library Imports
import os
import json

# Package Imports
import click

# Local Imports
from .cruncher import GIFCruncher, JPEGCruncher, JPEG2000Cruncher, PNGCruncher, WebPCruncher
from .utils import friendly_data_units

SUPPORTED_FILES_EXTENSIONS = ['bmp', 'dib', 'jpg', 'jpeg', 'gif', 'nef', 'tif', 'webp', 'png', 'ico', 'j2p', 'jpx']
OUTPUT_FILE_FORMATS = ['JPEG', 'PNG']
FUTURE_FORMATS = ['JPEG 2000', 'WebP', 'GIF']

ERROR_HANDLING = ["strict", "errors", "warnings", "silent"]


class CrunchHandler:

    def __init__(self, image, directory, output, file_format, quality, size,
                 append, orientation, aspect, metadata, nversions, recursive, config):
        self.image = image
        self.directory = directory
        self.output = output
        self.file_format = file_format
        self.quality = quality
        self.size = size
        self.append = append
        self.orientation = orientation
        self.aspect = aspect
        self.metadata = metadata
        self.settings = {}
        self.nversions = nversions
        self.recursive = recursive
        self.versions = []
        self.images = []
        self.directories = []
        self.config = config
        self.messages =[]

        self.mode = self.check_mode()

        self.get_output_dir(output, directory)
        self.generate_versions()
        self.get_images()
        self.build_output_directories()

        self.ncruches = self.get_num_crunches()

        self.input_bytes = 0
        self.output_bytes = 0

    def get_output_dir(self, output, directory):
        if output is None:
            self.output = os.path.join(directory, 'crunched')

    def generate_versions(self):
        if self.nversions > 1 and self.config is None:
            click.echo('Please enter the needed information for each version.')
            i = 1
            while i <= self.nversions:
                click.echo(f'Version {i}'),
                size = self.parse_size(click.prompt(f'Size WIDTH HEIGHT', type=str))
                self.versions.append({
                    'version': i,
                    'file_format': click.prompt(f'File format', type=click.Choice(OUTPUT_FILE_FORMATS),
                                                show_choices=False),
                    'width': size[0],
                    'height': size[1],
                    'quality': click.prompt(f'Quality',  type=click.IntRange(1, 100, clamp=True), default=80),
                    'append': click.prompt(f'Append filename',  type=str, default='', show_default=False),
                    'aspect': click.prompt(f'Keep aspect', type=bool, default=False),
                    'orientation': click.prompt(f'Keep orientation', type=bool, default=False),
                    'metadata': click.prompt(f'Keep Metadata', type=bool, default=False),
                })
                i += 1

        elif self.config:
            try:
                with open(self.config) as json_config:
                    configs = json.load(json_config)
                    self.parse_json_configs(configs)
            except FileNotFoundError:
                click.echo(f'Cruncher could not find the config file. Please make sure the path is correct.'),

        else:
            self.versions.append({
                'version': 1,
                'file_format': self.file_format,
                'width': self.parse_size(self.size)[0],
                'height': self.parse_size(self.size)[1],
                'quality': self.quality,
                'append': self.append,
                'aspect': self.aspect,
                'orientation': self.orientation,
                'metadata': self.metadata,
                'subsampling': -1
            })

    def get_images(self):
        if self.mode == 'img':
            self.images = [self.image]
        else:
            self.images = [image for image in self.scan_directory()]

    def scan_directory(self, path=None):
        """
        Finds all supported images in the given directory. If in recursive mode
        subdirectories are also checked.

        :param path: The path of the directory to scan.
        :return:
        """
        if not path:
            path = self.directory
        for node in os.scandir(path):
            if node.is_file(follow_symlinks=False) and node.name.split('.')[-1].lower() in SUPPORTED_FILES_EXTENSIONS:
                yield node.path
            elif self.recursive and node.is_dir(follow_symlinks=False) and node.path != self.output:
                self.directories.append(node.path)
                yield from self.scan_directory(node.path)

    def build_output_directories(self):
        if not os.path.isdir(self.output):
            os.makedirs(self.output)

        for directory in self.directories:
            new_dir = os.path.join(self.output, os.path.relpath(directory, self.directory))
            if not os.path.isdir(new_dir):
                os.makedirs(new_dir)

    def get_num_crunches(self):
        """
        Calculate the number of crunches to be completed. This is calculated
        by multiplying the number of images by the number of versions.

        :return: Total number of crunches.
        """
        return len(self.images) * len(self.versions)

    @staticmethod
    def parse_size(size):
        if isinstance(size, tuple):
            return size
        size = size.split(' ')
        return tuple([int(size[0]), int(size[1])])

    def parse_json_configs(self, configs):
        self.image = self.get_config(configs, 'image', self.image)
        self.directory = self.get_config(configs, 'directory', self.directory)
        self.output = self.get_config(configs, 'output', self.output)
        self.recursive = self.get_config(configs, 'recursive', self.recursive)
        self.settings['icc_conversions'] = self.get_config(configs, 'icc_conversions', None)
        self.versions = []
        versions = configs.get('versions')
        for version in versions:
            raw_enhance = self.get_config(version, 'enhance', None)
            enhance = {}
            if raw_enhance is not None:
                for prop in raw_enhance.keys():
                    if prop in ['brightness', 'color', 'contrast', 'sharpness', 'temperature']:
                        enhance[prop] = raw_enhance[prop]
            # click.echo(enhance)
            self.versions.append({
                'file_format': self.get_config(version, 'file_format', 'jpg'),
                'quality': self.get_config(version, 'quality', 80),
                'width': self.get_config(version, 'width'),
                'height': self.get_config(version, 'height'),
                'append': self.get_config(version, 'append'),
                'orientation': self.get_config(version, 'keep_orientation', False),
                'aspect': self.get_config(version, 'keep_aspect', False),
                'metadata': self.get_config(version, 'keep_metadata', False),
                'subsampling': self.get_config(version, 'subsampling', None),
                'icc_conversion': self.get_config(version, 'icc_conversion', None),
                'enhance': enhance
            })
        self.nversions = len(versions)
        self.mode = self.check_mode()

    @staticmethod
    def get_config(configs, setting, default=None):
        """
        Checks if the setting exists in the configs dictionary. If it exists
        it is returned. Otherwise the default is returned.

        :param configs: Configs dictionary from the JSON config file.
        :param setting: The setting to be checked.
        :param default: The default to return if there is no setting.
        :return: The final config setting.
        """
        if configs.get(setting):
            return configs.get(setting)
        else:
            return default

    def check_mode(self):
        """
        Check if Image or Directory mode should be used.

        :return: 'img' or 'dir'
        """
        if self.image:
            return 'img'
        else:
            return 'dir'

    def run_cruncher(self):
        with click.progressbar(self.images, len(self.images), "Crunching images") as images:
            for image in images:
                self.input_bytes += os.stat(image).st_size
                for version in self.versions:
                    if self.mode == 'img':
                        path = self.image
                    else:
                        path = self.directory
                    if version.get('file_format') == 'GIF':
                        cruncher = GIFCruncher(self.mode, path, self.output, image, version, self.settings)
                        self.output_bytes += cruncher.output_bytes
                        self.messages += cruncher.messages
                    if version.get('file_format') == 'JPEG':
                        cruncher = JPEGCruncher(self.mode, path, self.output, image, version, self.settings)
                        self.output_bytes += cruncher.output_bytes
                        self.messages += cruncher.messages
                    if version.get('file_format') == 'JPEG2000':
                        cruncher = JPEG2000Cruncher(self.mode, path, self.output, image, version, self.settings)
                        self.output_bytes += cruncher.output_bytes
                        self.messages += cruncher.messages
                    if version.get('file_format') == 'PNG':
                        cruncher = PNGCruncher(self.mode, path, self.output, image, version, self.settings)
                        self.output_bytes += cruncher.output_bytes
                        self.messages += cruncher.messages
                    if version.get('file_format') == 'WebP':
                        cruncher = WebPCruncher(self.mode, path, self.output, image, version, self.settings)
                        self.output_bytes += cruncher.output_bytes
                        self.messages += cruncher.messages

    def get_stats(self):
        """
        Build crunch statistics / information.
        :return: Dictionary: Stats data
        """
        images = len(self.images)
        versions = len(self.versions)
        new_images = self.ncruches
        input_bytes = self.input_bytes
        output_bytes = self.output_bytes
        percent_gain = round(100 - output_bytes * 100 / input_bytes, 2)
        average_gain = round(100 - (output_bytes / new_images) * 100 / (input_bytes / images), 2)

        stats = {
            'images': images,
            'versions': versions,
            'new_images': new_images,
            'input_bytes': friendly_data_units(input_bytes, 'B'),
            'output_bytes': friendly_data_units(output_bytes, 'B'),
            'percent_gain': percent_gain,
            'average_gain': average_gain,
        }
        return stats

    def print_messages(self):
        for message in self.messages:
            click.echo(f'\n{message}')



"""
:Project: Cruncher
:Contents: utils.py
:copyright: © 2019-2022 Daniel Morell
:license: MIT, see LICENSE for more details.
:Author: Daniel Morell
"""
# Standard Library Imports

# Package Imports

# Local Imports

UNITS = ['B', 'KB', 'MB', 'GB', 'TB']


def friendly_data_units(data, unit, digits=5):
    """
    Turns large number of byte units into larger units for easier reading.

    :param data: Integer|Float: The number of bytes.
    :param unit: String: The byte unit ['B', 'KB', 'MB', 'GB', 'TB'].
    :param digits: Integer: The number of decimal digits to round to.
    :return: 2 Tuple: New data quantity and unit string.
    """
    unit_num = UNITS.index(unit)
    if data > 1000 and len(UNITS) >= (unit_num + 1):
        unit = UNITS[unit_num + 1]
        data = data / 1000
        data, unit = friendly_data_units(data, unit)
    if digits:
        n = 5 - len(str(data).split('.')[0])
        if n > 3:
            n = 3
        data = round(data, n)
    return data, unit


def calculate_temperature_change(degree: int = 0) -> tuple:
    """
    Turns a degree in a range from -100 to 100 into a RGB color conversion.

    Negative degrees cool the image (increase blues)

    Positive degrees heat the image (increase reds)

    :param degree: Integer: The intended color temperature change
    """
    degree = min(100, degree) if degree >= 0 else max(-100, degree)
    red = 255 if degree > 1 else round(255 - (abs(degree) / 100 * 255))
    green = 255 if degree == 0 else round(255 - (abs(degree) / 100 * 255))
    blue = 255 if degree < 1 else round(255 - (abs(degree) / 100 * 255))
    return red, green, blue
